import os
import sys
from pathlib import Path

from loguru import logger

from .commons import env_to_bool
from .config import conf_dir

logdir = Path(conf_dir)
if not logdir.exists():
    os.mkdir(logdir)

stdout_level = "DEBUG" if env_to_bool(os.getenv("DEBUG")) else "WARNING"
file_level = "DEBUG" if env_to_bool(os.getenv("DEBUG")) else "INFO"

log_format = "{time}\t{message}"
logger.remove()
logger.add(sys.stdout, format=log_format, level=stdout_level, colorize=True, backtrace=True, diagnose=True)
logger.add(logdir.joinpath("peracotta.log"), format=log_format, level=file_level)

logger.info(f"{conf_dir = }")



from PyQt6 import QtWidgets

from peracotta.peralog import logger
from .exceptions import MissingFeaturesError

critical_errors = [MissingFeaturesError]  # error classes that should crash the program


def gui_excepthook(exc_type, exc_value, exc_tb):
    """Custom exception handler for peracotta's GUI version

    Args:
        exc_type: exception type
        exc_value: exception value
        exc_tb: exception traceback
    """
    QtWidgets.QMessageBox.warning(None, "Error", f"Peracotta encountered an exception ({exc_type.__name__}).\nSee logs for the traceback.")
    if any([exc_type is exc_t for exc_t in critical_errors]):
        logger.error("Encountered a critical error")
        QtWidgets.QApplication.quit()

    # These two lines are for pretty printing traceback with color
    # and additional info.
    # This is loguru syntax and should be modified if the logging system is changed
    options = ((exc_type, exc_value, exc_tb),) + logger._options[1:]
    logger._log("ERROR", False, options, "", None, None)



from cgitb import text
from PyQt6 import QtWidgets, QtCore, uic, QtGui
import json
from peracotta.constants import PATH


class JsonWidget(QtWidgets.QDialog):
    def __init__(self, data: list[dict], window_size: QtCore.QSize):
        super().__init__()
        layout = QtWidgets.QVBoxLayout()
        text_edit = QtWidgets.QPlainTextEdit()
        text_edit = QtWidgets.QTextEdit()
        text_edit.setWordWrapMode(QtGui.QTextOption.WrapMode.NoWrap)
        text_edit.setPlainText(f"{json.dumps(data, indent=2)}")
        text_edit.setReadOnly(True)
        layout.addWidget(text_edit)
        self.setLayout(layout)
        new_size = QtCore.QSize(int(window_size.width() * 0.8), int(window_size.height() * 0.8))
        self.resize(new_size)
        self.exec()


class ErrorDialog(QtWidgets.QDialog):
    def __init__(self, parent: QtWidgets.QMainWindow, title: str, detailed_error: str):
        super().__init__(parent)
        pass
        uic.loadUi(PATH["ERRORDIALOG"], self)
        self.setWindowTitle("Error")
        self.iconLabel = self.findChild(QtWidgets.QLabel, "iconLabel")
        self.textLabel = self.findChild(QtWidgets.QLabel, "textLabel")
        self.textLabel.setText(title)
        self.errorTextEdit = self.findChild(QtWidgets.QPlainTextEdit, "errorTextEdit")
        self.errorTextEdit.setPlainText(detailed_error)
        self.show()
